package ru.yarsu;

import io.pebbletemplates.pebble.PebbleEngine;

import java.io.IOException;
import java.io.StringWriter;
import java.time.DayOfWeek;
import java.time.LocalTime;
import java.util.Arrays;
import java.util.HashMap;

public class Main {
    public static void main(String[] args) {
        var schedule = formSchedule();
        var pebbleEngine = configureEngine();

        // Помещаем данные в первый шаблон
        generateDataFromSomePeb(schedule, pebbleEngine);
    }

    private static void generateDataFromSomePeb(
            ScheduleInfo scheduleInfo,
            PebbleEngine pebbleEngine
    ) {
        // Получаем доступ к шаблону
        var template = pebbleEngine.getTemplate("templates/some.peb");

        // Формируем контекст, который необходим шаблону для отображения данных
        var context = new HashMap<String, Object>();
        // Первый элемент из списка расписаний
        var itemInfo = scheduleInfo.scheduleItems().get(0);
        context.put("data", itemInfo);
        // Находим соответствующий ему курс по идентификатору
        var course = scheduleInfo.courses()
                .stream()
                .filter(course1 -> course1.id().equals(itemInfo.courseId()))
                .findFirst()
                .get(); // Опасная операция, но мы верим, что всё будет хорошо, так ведь?
        context.put("course", course);

        // Подготавливаем объект, в который будет происходить запись данных
        var writer = new StringWriter();
        try {
            // Записываем данные
            template.evaluate(writer, context);
        } catch (IOException exception) {
            System.err.println("Не могу обработать шаблон. Ниже представлена информация о шаблоне");
            System.err.println(exception.getMessage());
            return;
        }
        // Выводим успешный результат на стандартный поток вывода
        System.out.println(writer);
    }

    private static PebbleEngine configureEngine() {
        return new PebbleEngine.Builder().build();
    }

    private static ScheduleInfo formSchedule() {
        var physicsCourse = new Course(
                "physics-course",
                "Физика",
                "Иванов И. И.",
                "Курс про устройство мира, если на него смотреть с помощью современного аппарата"
        );
        var webCourse = new Course(
                "web-course",
                "Разработка веб-приложений",
                "Васильев А. М.",
                "Курс про разработку приложений для обслуживания людей с помощью браузера и интернета"
        );
        var courses = Arrays.asList(
                physicsCourse,
                webCourse
        );

        var schedule = Arrays.asList(
                new ScheduleItem(
                        physicsCourse.id(),
                        203,
                        DayOfWeek.MONDAY,
                        LocalTime.of(9, 0)
                ),
                new ScheduleItem(
                        physicsCourse.id(),
                        301,
                        DayOfWeek.MONDAY,
                        LocalTime.of(13, 30)
                ),
                new ScheduleItem(
                        webCourse.id(),
                        223,
                        DayOfWeek.WEDNESDAY,
                        LocalTime.of(13, 30)
                ),
                new ScheduleItem(
                        webCourse.id(),
                        220,
                        DayOfWeek.WEDNESDAY,
                        LocalTime.of(16, 30)
                )
        );

        return new ScheduleInfo(
                courses,
                schedule
        );
    }
}